\encoding{UTF-8}
\name{Harman}
\docType{data}
\alias{Harman}
\alias{Harman8}
\title{Harman's Eight Physical Variables}
\description{
  Unrotated factor loading matrix extracted by the centroid method
  for Harman's 8 physical variables, a classic dataset in factor analysis.
}
\usage{
  data(Harman)
}
\details{
  \code{Harman8} is a \eqn{8 \times 2}{8 x 2} matrix of unrotated
  factor loadings extracted by the centroid method (Table 14.3 in
  Harman, 1976) from the correlation matrix of 8 physical measurements
  taken on 305 girls (Table 2.3 in Harman, 1976). The centroid method
  was a standard extraction procedure before iterative methods became
  computationally feasible. The two columns represent unrotated centroid
  factors (\code{CF1} and \code{CF2}). Row names are the 8 variable
  names.

  The covariance matrix for these 8 physical variables is available
  in base R (see \code{\link[datasets]{Harman23.cor}}).
}
\format{
  \code{Harman8}: a \eqn{8 \times 2}{8 x 2} numeric matrix of
  unrotated centroid factor loadings. Row names are variable
  names; column names are \code{CF1} and \code{CF2}.
}
\source{
  Harman, H.H. (1976). \emph{Modern Factor Analysis} (3rd ed.).
  University of Chicago Press.
}
\seealso{
  \code{\link[datasets]{Harman23.cor}},
  \code{\link{GPForth}},
  \code{\link{rotations}},
  \code{\link{Thurstone}},
  \code{\link{WansbeekMeijer}},
  \code{\link{CCAI}}
}
\examples{
  data(Harman, package = "GPArotation")

  # Centroid unrotated loadings from Harman (1976) Table 14.3
  print(Harman8)

  # Rotate the centroid solution
  quartimax(Harman8)
  oblimin(Harman8, randomStarts = 100)
}
\keyword{datasets}