\name{00.GPArotation.Intro}
\alias{00.GPArotation.Intro}
\docType{package}

\title{GPA Rotation for Factor Analysis}

\description{See \code{\link{GPArotation-package}} ( in the help system use
   package?GPArotation or ?"GPArotation-package") for an overview.
}
\keyword{package}

