\name{plot.GPArotation}
\alias{plot.GPArotation}
\title{Plot Method for GPArotation Objects}
\description{
  Produces visual summaries of rotated factor matrices, including an APA-style 
  salient table-style, a heatmap, a tripartite pairs plot of factor geometry, 
  a macro-level profile bar chart, a target rotation schema, 
  a rotation trajectory view, or a set of diagnostics.
}
\usage{
  \method{plot}{GPArotation}(x, type = c("salient", "pairs", "profile", "vector", 
     "target", "heatmap", "trajectory", "diagnostics", "residuals"), 
     hide_hyperplane = FALSE, salient = 0.40, cutoff = .1, R = NULL, 
     factors = c(1,2), items = NULL, main = NULL, \dots)
     
  }
\arguments{
  \item{x}{An object of class \code{"GPArotation"}.}
 
  \item{type}{A character string indicating the type of plot. Must be one 
    of \code{"salient"}, \code{"pairs"}, or \code{"profile"}, \code{"vector"}, 
    \code{"target"}, \code{"heatmap"}, \code{"trajectory"}, \code{"diagnostics"},
    \code{residuals}.}

  \item{hide_hyperplane}{Boolean. Hide values below cutoff. Default is FALSE.}
  
  \item{salient}{Numeric. Absolute values equal to or above this are highlighted 
  as salient. Default is 0.40.}
 
  \item{cutoff}{Numeric. Absolute values strictly below this are considered 
    noise/hyperplane. Default is 0.1.}

  \item{R}{Correlation matrix. If not provided taken from GPArotation object. 
      Default is NULL.}

  \item{factors}{Two factors used in trajectory plot. Default = c(1,2).}

  \item{items}{items plotted in the trajectory plot. All items are 
     plotted by default. Default = NULL.}

  \item{main}{main title where applicable. Default = NULL.}
  
  \item{\dots}{Additional graphical parameters passed to base plotting functions.}
}
\value{
  Invisibly returns the original \code{x} object.
}
\author{
  Your Name
}
\examples{
  \dontrun{
    # Assuming 'res' is a fitted GPArotation object:
    plot(res, type = "salient")
    plot(res, type = "pairs")
    plot(res, type = "profile")
  }
}
