\encoding{UTF-8}
\name{WansbeekMeijer}
\docType{data}
\alias{WansbeekMeijer}
\alias{NetherlandsTV}
\title{Netherlands Television Viewership Data}
\description{
  Correlation matrix for Netherlands television viewership, used as an
  example dataset for factor rotation. From Wansbeek and Meijer (2000),
  page 171.
}
\usage{
  data(WansbeekMeijer)
}
\details{
  \code{NetherlandsTV} is a list with components \code{$cov}
  (a \eqn{7 \times 7}{7 x 7} correlation matrix for 7 television
  viewership variables measured in the Netherlands) and \code{$n.obs}
  (sample size, 2154). Use \code{cov2cor(NetherlandsTV\$cov)} to obtain
  the correlation matrix, or pass \code{NetherlandsTV} directly to
  \code{\link[stats]{factanal}} which handles the list structure
  automatically. It is used throughout the \code{GPArotation}
  documentation and vignettes as an example dataset for oblique rotation
  with 2 or 3 factors.
}
\format{
  \code{NetherlandsTV} is a list with components:
  \itemize{
    \item \code{$cov}: a \eqn{7 \times 7}{7 x 7} numeric correlation matrix
    \item \code{$n.obs}: sample size (2154)
  }
}
\source{
  Wansbeek, T. and Meijer, E. (2000). \emph{Measurement Error and
  Latent Variables in Econometrics}. North-Holland.
}
\seealso{
  \code{\link{GPForth}},
  \code{\link{rotations}},
  \code{\link{Thurstone}},
  \code{\link{Harman}},
  \code{\link{CCAI}}
}
\examples{
  data(WansbeekMeijer, package = "GPArotation")

  # Correlation matrix
  round(cov2cor(NetherlandsTV$cov), 2)

  # factanal picks up n.obs automatically from the list
  factanal(factors = 2, covmat = NetherlandsTV, rotation = "none")

  # Two-step oblique rotation
  fa.unrotated <- factanal(factors = 3, covmat = NetherlandsTV,
                           rotation = "none")
  oblimin(loadings(fa.unrotated), randomStarts = 100)
}
\keyword{datasets}