\encoding{UTF-8}
\name{Thurstone}
\docType{data}
\alias{Thurstone}
\alias{box26}
\title{Thurstone Box Data}
\description{
  Factor loading matrix derived from Thurstone's box data.
}
\usage{
  data(Thurstone)
}
\details{
  Loading the data makes the following object available:

  \code{box26} is a \eqn{26 \times 3}{26 x 3} unrotated factor loading
  matrix for 26 variables measured on a set of boxes. It appears
  frequently in the rotation literature as a dataset for
  comparing rotation criteria, particularly for assessing local minima.

  The matrix is suitable as input to \code{GPForth}, \code{GPFoblq},
  and all rotation wrapper functions in \code{GPArotation}. For a richer
  collection of factor analysis datasets, see the \code{psych} package.
}
\format{
  A numeric matrix:
  \itemize{
    \item \code{box26}: 26 x 3 matrix of unrotated factor loadings.
  }
}
\source{
  Thurstone, L.L. (1947). \emph{Multiple Factor Analysis}.
  University of Chicago Press.
}
\seealso{
  \code{\link{GPForth}},
  \code{\link{rotations}},
  \code{\link{Harman}},
  \code{\link{CCAI}},
  \code{\link{WansbeekMeijer}},
  \code{\link{GriffithMulaik}}  
}
\keyword{datasets}